#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>


#include "glcd.h"


#define F_CPU 16000000UL  // 4 MHz
#define SPIPORT PORTB
#define SPIDDR DDRB
#define CS 5
#define CLK 1
#define SDA 2
#define RESET 6
#define cbi(reg, bit) (reg&=~(1<<bit))
#define sbi(reg, bit) (reg|= (1<<bit))
#define CS0 cbi(SPIPORT,CS);
#define CS1 sbi(SPIPORT,CS);
#define CLK0 cbi(SPIPORT,CLK);
#define CLK1 sbi(SPIPORT,CLK);
#define SDA0 cbi(SPIPORT,SDA);
#define SDA1 sbi(SPIPORT,SDA);
#define RESET0 cbi(SPIPORT,RESET);
#define RESET1 sbi(SPIPORT,RESET);
#define byte unsigned char
byte n=0;
byte s1,s2;
byte r,g,b;
uint8_t color=0;
uint8_t xCoord=0;
uint8_t yCoord=0;
uint8_t memAccessCtrl=0;
uint8_t fontOffset=0;

font f;

//Funktionen
void lcdinit(void);
void sendCMD(byte cmd);
void sendData(byte cmd);
void shiftBits(byte b);
void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y);
void lcd_box(int lx1,int lx2,int ly1,int ly2,unsigned char color);
void clsdsp(unsigned char color);
void Lcd_window(int lx1,int lx2,int ly1,int ly2);
int pixel_lcd(int lx1,int lx2,int ly1,int ly2);
void grafik_lcd(int lx1,int lx2,int ly1,int ly2, unsigned char bild[]);
void window(int x1,int x2,int y1,int y2);
int n6100DrawChar(char c);
void n6100NewLine(void);
void n6100GotoXY(uint8_t x, uint8_t y);
void n6100SetDrawRect(uint8_t startX, uint8_t endX, uint8_t startY, uint8_t endY);
void n6100SetColor(uint8_t newColor);
void n6100SetFont(font newFont);
void n6100SetFontOffset(uint8_t newOffset) ;
void n6100DrawPixelXY(uint8_t x, uint8_t y);
void n6100DrawPixel(void);
void n6100DrawRect(uint8_t x, uint8_t y, uint8_t width, uint8_t height);
void n6100DrawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void n6100DrawCircle(uint8_t xCenter, uint8_t yCenter, uint8_t radius);

//DISPLAY TIMING
void waitms(int ms)
 {
  int i;
  for (i=0;i<ms;i++) _delay_ms(1);
}



//INIT DISPLAY
void lcdinit (void) 
{

  SPIDDR=(1<<SDA)|(1<<CLK)|(1<<CS)|(1<<RESET); //Port-Direction Setup


  
  int i;

  CS0
  SDA0
  CLK1

  RESET1
  RESET0
  waitms(10);
  RESET1

  CLK1
  SDA1
  CLK1

  sendCMD(0x01);
  waitms(10);
  sendCMD(0x11);
  waitms(40);
  sendCMD(0x29);
  waitms(40);
  sendCMD(0x3a);
  sendData(0x02);
  sendCMD(0x20);
  sendCMD(0x36);
  sendData(0x00);
  sendCMD(0x2d);

//FARBTABELLE 254
unsigned char lcdinitd[]={0x00 , 0x02 , 0x04 , 0x06 , 0x09 , 0x0B , 0x0D , 0x0F , 0x00 , 0x02 , 0x04 , 0x06 , 0x09 , 0x0B , 0x0D , 0x0F , 0x00 , 0x04 , 0x0B , 0x0F};

for (i=0;i<22;i++)
{
sendData(lcdinitd[i]);

}
/*sendCMD(MEM_ACCESS_CTRL);
	memAccessCtrl = (0x01 << MIRROR_X) | (0x01 << RGB);
	sendData(memAccessCtrl);
	*/
//DISPLAY LSCHEN
 clsdsp(weiss);  

}


//Display Clear komplett
void clsdsp(unsigned char color)
{  
lcd_box(0,131,0,131,color);
}


void grafik_lcd(int lx1,int lx2,int ly1,int ly2,unsigned char bild[])
{
int i;
int pixel;
Lcd_window(lx1,lx2,ly1,ly2);
pixel=pixel_lcd(lx1,lx2,ly1,ly2);
for(i=1500;i<pixel;i++)
{
sendData(bild[i]);
}



}

void n6100DrawCircle(uint8_t xCenter, uint8_t yCenter, uint8_t radius) {
  	int16_t tSwitch, y, x = 0;
  	uint8_t d;

  	d = yCenter - xCenter;
  	y = radius;
  	tSwitch = 3 - 2 * radius;
  	
  	while (x <= y) {
    	n6100DrawPixelXY(xCenter + x, yCenter + y);
    	n6100DrawPixelXY(xCenter + x, yCenter - y);
    	
    	n6100DrawPixelXY(xCenter - x, yCenter + y);
    	n6100DrawPixelXY(xCenter - x, yCenter - y);
    	
    	n6100DrawPixelXY(yCenter + y - d, yCenter + x);
    	n6100DrawPixelXY(yCenter + y - d, yCenter - x);
    	
    	n6100DrawPixelXY(yCenter - y - d, yCenter + x);
    	n6100DrawPixelXY(yCenter - y - d, yCenter - x);

    	if (tSwitch < 0) {
    		tSwitch += (4 * x + 6);
    	} else {
      		tSwitch += (4 * (x - y) + 10);
      		y--;
    	}
    	x++;
  	}
}


void n6100DrawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2) {
	uint8_t length, xTmp, yTmp, i, y, yAlt;
	int16_t m;
	
	if(x1 == x2) {									// vertical line
		// x1|y1 must be the upper point
		if(y1 > y2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = y2-y1;
		for(i=0; i<=length; i++) {
			n6100DrawPixelXY(x1, y1+i);
		}
	} else if(y1 == y2) {							// horizontal line
		// x1|y1 must be the left point
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = x2-x1;
		for(i=0; i<=length; i++) {
			n6100DrawPixelXY(x1+i, y1);
		}
	} else {
		// x1 must be smaller than x2
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1)) {	// angle larger or equal 45
			length = x2-x1;								// not really the length :)
			m = ((y2-y1)*200)/length;
			yAlt = y1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+y1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				n6100DrawLine(x1+i, yAlt, x1+i, y);
				if(length<=(y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length<=(y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		} else {										// angle smaller 45
			// y1 must be smaller than y2
			if(y1 > y2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}
			length = y2-y1;
			m = ((x2-x1)*200)/length;
			yAlt = x1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+x1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				n6100DrawLine(yAlt, y1+i, y, y1+i);
				if(length<=(x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length<=(x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		}
	}
}


void n6100DrawPixel(void) {
	sendData(color);
}
void n6100DrawPixelXY(uint8_t x, uint8_t y) {
	n6100GotoXY(x, y);
	sendCMD(MEM_WRITE);
	sendData(color);
}

void n6100SetDrawRect(uint8_t startX, uint8_t endX, uint8_t startY, uint8_t endY) {
	sendCMD(SET_X_ADDR);
	sendData(startX+1);
	sendData(endX+1);
	
	sendCMD(SET_Y_ADDR);
	sendData(startY+1);
	sendData(endY+1);
	
	xCoord=startX;
	yCoord=startY;
}

void n6100DrawRect(uint8_t x, uint8_t y, uint8_t width, uint8_t height) {
	width--;
	height--;
	n6100DrawLine(x, y, x+width, y);				// top
	n6100DrawLine(x, y, x, y+height);				// left
	n6100DrawLine(x, y+height, x+width, y+height);	// bottom
	n6100DrawLine(x+width, y, x+width, y+height);	// right
}


void n6100GotoXY(uint8_t x, uint8_t y) {
	n6100SetDrawRect(x, DISP_X_END, y, DISP_Y_END);
}
void n6100SetColor(uint8_t newColor) {
	color = newColor;
}

void n6100SetFont(font newFont) {
	f = newFont;
}

void n6100SetFontOffset(uint8_t newOffset) {
	fontOffset = newOffset;
}

void n6100NewLine(void) {
	if(yCoord+f.height < 128)
		n6100GotoXY(fontOffset, yCoord+f.height);
	else
		n6100GotoXY(fontOffset, 0);
}


int n6100DrawChar(char c) {
	uint16_t index;
	uint8_t i, j, w, data, skipped=1, startY=yCoord;
	
	sendCMD(MEM_ACCESS_CTRL);
	memAccessCtrl |= (0x01 << VERT_WRITE);
	sendData(memAccessCtrl);
	
	if(c == '\n')
		n6100NewLine();
	if(c < 32)
		return 0;
	
	c -= 32;
	index = c*f.width*(f.height/8);
	
	for(w=0; w<f.width; w++) {
		for(i=0; i<f.height/8; i++) {
			data = pgm_read_byte(f.charData + index++);
			for(j=0; j<8; j++) {
				if(data & 0x01) {
					if(skipped) {
						n6100GotoXY(xCoord, yCoord);
						sendCMD(MEM_WRITE);
						skipped=0;
					}
					n6100DrawPixel();
					yCoord++;
				} else {
					skipped=1;
					n6100GotoXY(xCoord, ++yCoord);
				}
				data >>= 1;
			}
		}
		n6100GotoXY(++xCoord, startY);
	}
	
	sendCMD(MEM_ACCESS_CTRL);
	memAccessCtrl &= ~(0x01 << VERT_WRITE);
	sendData(memAccessCtrl);
	
	return 0;
}








//Display Fenster definieren
void lcd_box(int lx1,int lx2,int ly1,int ly2,unsigned char color)
{
Lcd_window(lx1,lx2,ly1,ly2);
int px;
int py;
int pixel;
int i;
   px = lx2 - lx1;
   px++; 
   py = ly2 - ly1;
   py++;
   pixel = px * py;
   for (i= 0;i<=pixel;i++)
   {
   sendData(color);

   }



}


//Fenster AdressBereich setzen
void Lcd_window(int lx1,int lx2,int ly1,int ly2)
{  sendCMD(0x2A);
   sendData(lx1);
   sendData(lx2);
   sendCMD(0x2B);
   sendData(ly1);
   sendData(ly2);
   sendCMD(0x2C);
  
}


void window(int x1,int x2,int y1,int y2)
{
 sendCMD(0x2A);
 sendData(x1);
 sendData(x2);
 sendCMD(0x2B); 
 sendData(y1);
 sendData(y2);  
 sendCMD(0x2C);
   

}







void shiftBits(byte b) {

  CLK0
  if ((b&128)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&64)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&32)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&16)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&8)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&4)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&2)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&1)!=0) SDA1 else SDA0
  CLK1

}


//send data
void sendData(byte data) {

  CLK0
  SDA1                                                 //1 for param
  CLK1

  shiftBits(data);
}

//send cmd
void sendCMD(byte data) {

  CLK0
  SDA0                                                 //1 for cmd
  CLK1

  shiftBits(data);
}

int pixel_lcd(int lx1,int lx2,int ly1,int ly2)
{
int px;
int py;
px = lx2 - lx1;
   px++; 
   py = ly2 - ly1;
   py++;
   return( px * py);


}




//__attribute__ ((section(".eeprom")))
